@@ -3,9 +3,12 @@
 // SPDX-License-Identifier: LGPL-3.0-only
 
 #include <rz_cons.h>
-#include <rz_regex.h>
+#include <rz_util/rz_regex.h>
 #include <rz_util.h>
 #include "pager_private.h"
+#include "rz_vector.h"
+
+#define I(x) rz_cons_singleton()->x
 
 RZ_API int rz_cons_less_str(const char *str, const char *exitkeys) {
 	rz_return_val_if_fail(str && *str, 0);
@@ -29,7 +32,7 @@ RZ_API int rz_cons_less_str(const char *str, const char *exitkeys) {
 	RzRegex *rx = NULL;
 	int w, h, ch, to, ui = 1, from = 0, i;
 	const char *sreg;
-	RzList **mla;
+	RzPVector **mla;
 
 	// rcons kills str after flushing the buffer, so we must keep a copy
 	char *ostr = strdup(str);
@@ -45,7 +48,7 @@ RZ_API int rz_cons_less_str(const char *str, const char *exitkeys) {
 	if (lines_count < 1) {
 		mla = NULL;
 	} else {
-		mla = calloc(lines_count, sizeof(RzList *));
+		mla = calloc(lines_count, sizeof(RzPVector *));
 		if (!mla) {
 			free(p);
 			free(ostr);
@@ -53,9 +56,6 @@ RZ_API int rz_cons_less_str(const char *str, const char *exitkeys) {
 			return 0;
 		}
 	}
-	for (i = 0; i < lines_count; i++) {
-		mla[i] = rz_list_new();
-	}
 	rz_cons_set_raw(true);
 	rz_cons_show_cursor(false);
 	rz_cons_reset();
@@ -73,7 +73,7 @@ RZ_API int rz_cons_less_str(const char *str, const char *exitkeys) {
 		ch = rz_cons_readchar();
 		if (exitkeys && strchr(exitkeys, ch)) {
 			for (i = 0; i < lines_count; i++) {
-				rz_list_free(mla[i]);
+				rz_pvector_free(mla[i]);
 			}
 			free(p);
 			free(mla);
@@ -119,15 +119,15 @@ RZ_API int rz_cons_less_str(const char *str, const char *exitkeys) {
 			break;
 		case '/': /* search */
 			rz_cons_reset_colors();
-			rz_line_set_prompt("/");
-			sreg = rz_line_readline();
+			rz_line_set_prompt(I(line), "/");
+			sreg = rz_line_readline(I(line));
 			from = RZ_MIN(lines_count - 1, from);
 			/* repeat last search if empty string is provided */
 			if (sreg[0]) { /* prepare for a new search */
 				if (rx) {
 					rz_regex_free(rx);
 				}
-				rx = rz_regex_new(sreg, "");
+				rx = rz_regex_new(sreg, RZ_REGEX_EXTENDED | RZ_REGEX_MULTILINE, 0);
 			} else { /* we got an empty string */
 				from = pager_next_match(from, mla, lines_count);
 				break;
@@ -136,9 +136,12 @@ RZ_API int rz_cons_less_str(const char *str, const char *exitkeys) {
 				break;
 			}
 			/* find all occurrences */
-			if (pager_all_matches(p, rx, mla, lines, lines_count)) {
-				from = pager_next_match(from, mla, lines_count);
+			RzPVector *matches = rz_regex_match_all_not_grouped(rx, str, RZ_REGEX_ZERO_TERMINATED, 0, RZ_REGEX_DEFAULT);
+			if (rz_pvector_empty(matches)) {
+				rz_pvector_free(matches);
+				break;
 			}
+			from = pager_next_match(from, mla, lines_count);
 			break;
 		case 'n': /* next match */
 			/* search already performed */
@@ -155,7 +158,7 @@ RZ_API int rz_cons_less_str(const char *str, const char *exitkeys) {
 		}
 	}
 	for (i = 0; i < lines_count; i++) {
-		rz_list_free(mla[i]);
+		rz_pvector_free(mla[i]);
 	}
 	free(mla);
 	rz_regex_free(rx);
