@@ -43,6 +43,7 @@ RZ_IPI bool rz_core_visual_esil(RzCore *core) {
 	RzAnalysisOp aop;
 	ut8 buf[sizeof(ut64)];
 	unsigned int addrsize = rz_config_get_i(core->config, "esil.addr.size");
+	RzLine *line = core->cons->line;
 
 	if (core->blocksize < sizeof(ut64)) {
 		return false;
@@ -78,7 +79,7 @@ RZ_IPI bool rz_core_visual_esil(RzCore *core) {
 			const char *expr = rz_strbuf_get(&aop.esil);
 			rz_cons_printf(Color_RESET "esil: %s\n" Color_RESET, expr);
 			int wp = wordpos(expr, x);
-			char *pas = strdup(rz_str_pad(' ', wp ? wp + 1 : 0));
+			char *pas = rz_str_pad(' ', wp ? wp + 1 : 0);
 			int wp2 = wordpos(expr, x + 1);
 			free(word);
 			word = rz_str_ndup(expr + (wp ? (wp + 1) : 0), (wp2 - wp) - (wp ? 1 : 0));
@@ -90,10 +91,10 @@ RZ_IPI bool rz_core_visual_esil(RzCore *core) {
 				free(pas);
 				continue;
 			}
-			const char *pad = rz_str_pad('-', wp2 - ((wp > 0) ? wp + 1 : 0));
+			char *pad = rz_str_pad('-', wp2 - ((wp > 0) ? wp + 1 : 0));
 			rz_cons_printf(Color_RESET "      %s%s\n" Color_RESET, pas, pad);
 			free(pas);
-			// free (pad);
+			free(pad);
 		}
 		rz_cons_printf("esil regs:\n");
 		showreg(esil, "$$", "address");
@@ -160,7 +161,7 @@ RZ_IPI bool rz_core_visual_esil(RzCore *core) {
 			rz_cons_show_cursor(true);
 			rz_cons_set_raw(0);
 			*cmd = 0;
-			rz_line_set_prompt(":> ");
+			rz_line_set_prompt(line, ":> ");
 			if (rz_cons_fgets(cmd, sizeof(cmd), 0, NULL) < 0) {
 				cmd[0] = '\0';
 			}
